# -*- coding: utf-8 -*-
from core.libs import *

HOST = 'https://hdfull.pro'

LNG = Languages({
    Languages.sub_es: ['sub', 'espsub'],
    Languages.sub_en: ['engsub'],
    Languages.es: ['spa', 'esp'],
    Languages.en: ['eng'],
    Languages.la: ['lat']
})

QLT = Qualities({
    Qualities.rip: ['dvdrip', 'rhdtv'],
    Qualities.hd_full: ['hd1080m', 'hd1080'],
    Qualities.hd: ['hd720m', 'hdtv', 'hd720'],
    Qualities.scr: ['dvdscr', 'ts', 'cam']
})


def mainlist(item):
    logger.trace()

    itemlist = list()
    if login():
        new_item = item.clone(
            type='label',
            label="Películas",
            url=HOST,
            category='movie',
            thumb='thumb/movie.png',
            icon='icon/movie.png',
            poster='poster/movie.png'
        )
        itemlist.append(new_item)
        itemlist.extend(menupeliculas(new_item))

        new_item = item.clone(
            type='label',
            label="Series",
            url=HOST,
            category='tvshow',
            thumb='thumb/tvshow.png',
            icon='icon/tvshow.png',
            poster='poster/tvshow.png',
        )
        itemlist.append(new_item)
        itemlist.extend(menuseries(new_item))

    else:
        itemlist.append(Item(
            label="Es necesaria una cuenta en HDFull para poder usar este canal...",
            category='all',
            type='warning'
        ))

    itemlist.append(item.clone(
        action="config",
        label="Configuración",
        folder=False,
        category='all',
        type='setting'
    ))

    return itemlist


def config(item):
    v = platformtools.show_settings(item=item)
    platformtools.itemlist_refresh()
    return v


def login():
    logger.trace()
    if not settings.get_setting('user', __file__) or not settings.get_setting('password', __file__):
        return False
    data = httptools.downloadpage(HOST).data

    patron = '<input type=\'hidden\' name=\'__csrf_magic\' value="([^"]+)" />'
    sid = scrapertools.find_single_match(data, patron)
    post = {
        '__csrf_magic': sid,
        'username': settings.get_setting('user', __file__),
        'password': settings.get_setting('password', __file__),
        'action': 'login'
    }

    data = httptools.downloadpage(HOST, post=post).data

    if 'Bienvenido %s' % settings.get_setting('user', __file__) not in data:
        platformtools.dialog_notification('HDFull', 'Login incorrecto')
        return False
    else:
        return True


def menuusuario(item):
    logger.trace()
    itemlist = list()

    itemlist.append(Item(
        label="Peliculas",
        type='label',
        icon='icon/movie.png'
    ))

    itemlist.append(item.clone(
        action="user_movies",
        label="Favoritas",
        url=HOST + "/a/my?target=movies&action=favorite",
        content_type='movies',
        group=True,
        type="item"
    ))

    itemlist.append(item.clone(
        action="user_movies",
        label="Pendientes",
        url=HOST + "/a/my?target=movies&action=pending",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_movies",
        label="Vistas",
        url=HOST + "/a/my?target=movies&action=seen",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(Item(label="Series",
                         type='label',
                         icon='icon/tvshow.png'
                         ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Para ver",
        url=HOST + "/a/my?target=shows&action=watch",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Siguiendo",
        url=HOST + "/a/my?target=shows&action=following",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Favoritas",
        url=HOST + "/a/my?target=shows&action=favorite",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Pendientes",
        url=HOST + "/a/my?target=shows&action=pending",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Recomendadas",
        url=HOST + "/a/my?target=shows&action=recommended",
        content_type='movies',
        group=True,
        type='item'
    ))

    itemlist.append(item.clone(
        action="user_tvshows",
        label="Finalizadas",
        url=HOST + "/a/my?target=shows&action=seen",
        content_type='movies',
        group=True,
        type='item'
    ))

    return itemlist


def menupeliculas(item):
    logger.trace()
    itemlist = list()

    itemlist.append(item.clone(
        action="movies",
        label="ABC",
        url=HOST + "/peliculas/abc",
        type="item",
        group=True,
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="movies",
        label="Nuevas",
        url=HOST + "/peliculas/date",
        type="item",
        group=True,
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="movies",
        label="Actualizadas",
        url=HOST + "/peliculas-actualizadas",
        type="item",
        group=True,
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="movies",
        label="Rating IMDB",
        url=HOST + "/peliculas/imdb_rating",
        type="item",
        group=True,
        content_type='movies'
    ))

    itemlist.append(item.clone(
        action="generos",
        label="Géneros",
        url=HOST,
        type="item",
        group=True
    ))

    itemlist.append(item.clone(
        action="movie_search",
        label="Buscar",
        url=HOST,
        query=True,
        type='search',
        group=True,
        content_type='movies'
    ))

    return itemlist


def menuseries(item):
    logger.trace()

    itemlist = list()

    itemlist.append(item.clone(
        action="tvshows",
        label="Nuevas",
        url=HOST + "/series",
        type="item",
        group=True,
        content_type='tvshows'
    ))

    itemlist.append(item.clone(
        action="tvshows",
        label="Rating IMDB",
        url=HOST + "/series/imdb_rating",
        type="item",
        group=True,
        content_type='tvshows'
    ))

    itemlist.append(item.clone(
        action="series_az",
        type="item",
        group=True,
        label="ABC"
    ))

    itemlist.append(item.clone(
        action="generos_tvshow",
        label="Géneros",
        type="item",
        group=True,
        url=HOST
    ))

    itemlist.append(item.clone(
        action="tv_search",
        label="Buscar",
        url=HOST,
        query=True,
        type='search',
        group=True,
        content_type='tvshows'
    ))

    return itemlist


def generos(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url).data
    data = scrapertools.find_single_match(data, '<b class="caret"></b>  Películas</a>(.*?)</ul>')

    patron = '<li><a href="([^"]+)">([^<]+)</a></li>'

    matches = re.compile(patron, re.DOTALL).findall(data)

    for url, genre in matches:
        itemlist.append(item.clone(
            action='movies',
            label=genre,
            url=urllib_parse.urljoin(item.url, url),
            content_type='movies'
        ))

    return itemlist


def generos_tvshow(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url).data
    data = scrapertools.find_single_match(data, '<b class="caret"></b>  TV</a>(.*?)</ul>')

    patron = '<li><a href="([^"]+)">([^<]+)</a></li>'

    matches = re.compile(patron, re.DOTALL).findall(data)

    for url, genre in matches:
        itemlist.append(item.clone(
            action='tvshows',
            label=genre,
            url=urllib_parse.urljoin(item.url, url),
            content_type='tvshows'
        ))

    return itemlist


def movies(item):
    logger.trace()
    itemlist = list()


    data = httptools.downloadpage(item.url).data

    patron = r'<a href="(?P<url>[^"]+)" class="spec-border-ie".*?' \
             r'<img.*?src="(?P<thumb>[^"]+)".*?title="(?P<title>[^"]+)".*?' \
             r'(?P<lang>(?:<img src="[^<]+)+<\/div>)[^<]+<div class="right">.*?' \
             r'(?:<div class="rating">(?P<rat>[^<])+.*?>(?P<dec>[^<]+)<.*?)?setFavorite\(\d,\s*(?P<id>\d+)\s*,'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action='findvideos',
            url=urllib_parse.urljoin(item.url, result.group('url')),
            poster=result.group('thumb').replace("/tthumb/130x190/", "/tthumb/130x190/"),
            lang=[LNG.get(l) for l in
                  scrapertools.find_multiple_matches(result.group('lang'), 'src=".*?/([^./]+).png"')],
            rating='%s.%s' % (result.group('rat') or '0', result.group('dec') or '0'),
            title=result.group('title'),
            type='movie',
            content_type='servers',
            hdfullid=result.group('id'),
            hdfulltype='movies'
        ))

    # Paginador
    next_url = scrapertools.find_single_match(data, '<a href="([^"]+)">»</a>')
    if next_url:
        itemlist.append(item.clone(
            action=item.action,
            url=urllib_parse.urljoin(item.url, next_url),
            type='next'
        ))

    return itemlist


def tvshows(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url).data

    patron = r'<a href="(?P<url>[^"]+)" class="spec-border-ie".*?' \
             r'<img.*?src="(?P<thumb>[^"]+)".*?title="(?P<title>[^"]+)".*?' \
             r'(?:<div class="rating">(?P<rat>[^<])+.*?>(?P<dec>[^<]+)<.*?)?' \
             r'setFavorite\(\d,\s*(?P<id>\d+)\s*,'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action='seasons',
            url=urllib_parse.urljoin(item.url, result.group('url')),
            poster=result.group('thumb').replace("/tthumb/130x190/", "/tthumb/130x190/"),
            type='tvshow',
            content_type='seasons',
            rating='%s.%s' % (result.group('rat') or '0', result.group('dec') or '0'),
            title=result.group('title'),
            hdfullid=result.group('id'),
            hdfulltype='shows'
        ))

    # Paginador
    next_url = scrapertools.find_single_match(data, '<a href="([^"]+)">»</a>')
    if next_url:
        itemlist.append(item.clone(action='tvshows',
                                   url=urllib_parse.urljoin(item.url, next_url),
                                   type='next'
                                   ))
    return itemlist


def seasons(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url, add_referer=True).data
    patron = '<a href=\'(?P<url>[^\']+)\' rel="bookmark".*?' \
             '<img.*?original-title="(?P<title>[^"]+)".*?src="(?P<thumb>[^"]+)" />'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action='episodes',
            url=urllib_parse.urljoin(item.url, result.group('url')),
            poster=result.group('thumb').replace("/tthumb/130x190/", "/tthumb/130x190/"),
            type='season',
            content_type='episodes',
            tvshowtitle=item.title,
            season=int(result.group('title').split(' ')[1])
        ))

    return itemlist


def episodes(item):
    logger.trace()
    itemlist = list()

    post = {
        'action': 'season',
        'show': item.hdfullid,
        'season': item.season
    }

    episode_list = jsontools.load_json(
        httptools.downloadpage(
            urllib_parse.urljoin(item.url, '/a/episodes'),
            post=post,
            headers={'Referer': item.url}
        ).data
    )
    if not episode_list:
        itemlist.append(Item(type='no_content'))

    for episode in episode_list:
        itemlist.append(item.clone(
            action='findvideos',
            url=urllib_parse.urljoin(item.url + '/', 'episodio-%s' % episode['episode']),
            thumb=urllib_parse.urljoin(HOST, '/tthumb/130x190/' + episode['thumbnail']),
            lang=[LNG.get(l.lower()) for l in episode['languages']],
            type='episode',
            content_type='servers',
            episode=int(episode['episode']),
            title=str(episode['title'].get('es', episode['title']['en'])).strip('" '),
            hdfulltype='episodes',
            hdfullid=episode['id']
        ))

    return itemlist


def newest_episodes(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(item.url).data
    data = re.sub(r"\n|\r|\t|\s{2}|&nbsp;", "", data)

    patron = '<div class="item" style="text-align:center">.*?'
    patron += '<a href="(?P<url>[^"]+)" class=".*?'
    patron += 'src="(?P<thumb>[^"]+).*?'
    patron += '<div class="left">(?P<lang>.*?)<div class="right">.*?'
    patron += '<div class="rating">(?P<season>[^<]+).*?'
    patron += '<b class="sep">x</b>(?P<episode>[^<]+).*?'
    patron += 'title="(?P<title>[^"]+).*?'
    patron += 'data-seen="(?P<id>[^"]+)'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        tvshowstitle = result.group('title').split('-', 1)[0].strip()
        new_item = item.clone(
            action='findvideos',
            tvshowtitle=tvshowstitle,
            type='episode',
            content_type='servers',
            url=result.group('url'),
            thumb=scrapertools.find_single_match(result.group('thumb'), 'src=([^&]+)'),
            lang=[LNG.get(l) for l in
                  scrapertools.find_multiple_matches(result.group('lang'), 'src=".*?/([^./]+).png"')],
            season=int(result.group('season')),
            episode=int(result.group('episode')),
            hdfulltype='episodes',
            hdfullid=result.group('id')
        )
        itemlist.append(new_item)

    return itemlist


def tv_search(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(HOST).data

    post = {
        '__csrf_magic': scrapertools.find_single_match(data, '.__csrf_magic. value="(sid:[^"]+)"'),
        'menu': 'search',
        'query': item.query
    }
    data = httptools.downloadpage(HOST + '/buscar', post=post, headers={
        'Referer': HOST + '/buscar'
    }).data

    patron = r'<a href="(?P<url>[^"]+)" class="spec-border-ie".*?' \
             r'<img class=".*?show-thumbnail".*?src="(?P<thumb>[^"]+)".*?' \
             r'(?:<div class="rating">(?P<rat>[^<])+.*?>(?P<dec>[^<]+)<.*?)?' \
             r'title="(?P<title>[^"]+)".*?setFavorite\(\d,\s*(?P<id>\d+)\s*,'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action='seasons',
            url=urllib_parse.urljoin(item.url, result.group('url')),
            poster=result.group('thumb').replace("/tthumb/130x190/", "/tthumb/130x190/"),
            type='tvshow',
            content_type='seasons',
            rating='%s.%s' % (result.group('rat') or '0', result.group('dec') or '0'),
            title=result.group('title'),
            hdfullid=result.group('id'),
            hdfulltype='shows'
        ))

    return itemlist


def movie_search(item):
    logger.trace()
    itemlist = list()

    data = httptools.downloadpage(HOST).data
    post = {
        '__csrf_magic': scrapertools.find_single_match(data, '.__csrf_magic. value="(sid:[^"]+)"'),
        'menu': 'search',
        'query': item.query
    }

    data = httptools.downloadpage(HOST + '/buscar', post=post, headers={
        'Referer': HOST + '/buscar'
    }).data

    patron = r'<a href="(?P<url>[^"]+)" class="spec-border-ie".*' \
             r'?<img class=".*?spec-border".*?src="(?P<thumb>[^"]+)".*?' \
             r'(?:<div class="rating">(?P<rat>[^<])+.*?>(?P<dec>[^<]+)<.*?)?' \
             r'title="(?P<title>[^"]+)".*?setFavorite\(\d,\s*(?P<id>\d+)\s*,'

    for result in re.compile(patron, re.DOTALL).finditer(data):
        itemlist.append(item.clone(
            action='findvideos',
            url=urllib_parse.urljoin(item.url, result.group('url')),
            poster=result.group('thumb').replace("/tthumb/130x190/", "/tthumb/130x190/"),
            type='movie',
            content_type='servers',
            rating='%s.%s' % (result.group('rat') or '0', result.group('dec') or '0'),
            title=result.group('title'),
            hdfullid=result.group('id'),
            hdfulltype='movies'
        ))

    return itemlist


def series_az(item):
    logger.trace()

    itemlist = list()

    for letra in "ABCDEFGHIJKLMNOPQRSTUVWXYZ#":
        itemlist.append(item.clone(
            action='tvshows',
            label=letra,
            url=urllib_parse.urljoin(HOST, '/series/abc/' + letra.replace('#', '9')),
            content_type='tvshows'
        ))

    return itemlist


def findvideos(item):
    logger.trace()

    itemlist = list()
    import base64

    data = httptools.downloadpage(item.url, add_referer=True).data
    data_obf = scrapertools.find_single_match(data, 'var ad = \'([^\']+)\';')
    decoded = base64.b64decode(data_obf)

    servers = jsontools.load_json(obfs(decoded, 126 - get_key()))
    if not servers:
        servers = jsontools.load_json(obfs(decoded, 126 - get_key(True)))

    if not servers:
        itemlist = [Item(type='no_content')]

    decoder = Decoder()
    for server in servers:
        data = decoder.get(server['provider'], server['code'])
        itemlist.append(item.clone(
            action='play',
            url=data['link'],
            type='server',
            lang=LNG.get(server['lang'].lower()),
            quality=QLT.get(server['quality']),
            stream=data['stream']
        ))

    itemlist = servertools.get_servers_itemlist(itemlist)

    return itemlist


def obfs(data, key, n=126):
    data = six.ensure_text(data)
    chars = list(data)

    for i in range(0, len(chars)):
        c = ord(chars[i])
        if c <= n:
            number = (ord(chars[i]) + key) % n
            chars[i] = chr(number)

    return "".join(chars)


def get_key(renew=False):
    key = settings.get_setting('key', __file__)

    if not key or renew:
        data_js = httptools.downloadpage(HOST + "/templates/hdfull/js/jquery.hdfull.view.min.js").data
        key = scrapertools.find_single_match(data_js, r'JSON[^}]+atob[^}]+\(([^)]+)\)\);')
        settings.set_setting('key', key, __file__)

    if '0x' in key:
        return int(key, 16)

    return int(key)


class Decoder:
    def __init__(self):
        self.witdh = '920'
        self.height = '360'
        self.code = httptools.downloadpage(HOST + '/js/providers.js').data
        self.code = re.sub(r'\\x([0-F]+)', lambda x: chr(int(x.group(1), 16)), self.code)
        self.varname, values, self.code = re.compile(r'var (_0x[0-f]+)=(\[.*?\]);(.*)').findall(self.code)[0]
        self.values = eval(values)
        self.code = re.sub(r'%s\[([\d]+)\]' % self.varname, lambda x: repr(self.values[int(x.group(1))]), self.code)
        self.result = {}
        self.compose()

    def compose(self):
        result = {}

        for code, prov in re.compile(r"p\[(\d+)\]\s*=\s*{(.*?)};").findall(self.code):
            t = re.compile(r'[\'"]t[\'"]:([^,]+)').findall(prov)[0]
            link = re.compile(r'[\'"]l[\'"]:function.*?return \'(.*?)\'').findall(prov)[0] + '%s'
            result[str(eval(code))] = {
                'stream': True if t == "'s'" else False,
                'link': link
            }

        self.result = result

    def get(self, index, video_id):
        import copy
        value = copy.deepcopy(self.result[index])
        value['link'] = value['link'] % video_id
        return value
