# -*- coding: utf-8 -*-
from core.libs import *
import json
import six

locker = Lock()


def load_file(path):
    with locker:
        data = open(path, 'rb').read()

    data = load_json(six.ensure_str(data))
    return data


def dump_file(data, path):
    if not os.path.exists(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
    data = six.ensure_binary(dump_json(data))
    with locker:
        open(path, 'wb').write(data)


def load_js(*args, **kwargs):
    data = args[0]; args = args[1:] if args else kwargs.pop('s')
    data = re.sub(r'([\s{,[])\'?(\w+)\'?(\s*:)', r'\1"\2"\3', data)
    data = re.sub(r'(:\s*)\'(.*?)\'(\s*[}\],])', r'\1"\2"\3', data)
    return load_json(data, *args, **kwargs)


def load_json(*args, **kwargs):
    if "object_hook" not in kwargs:
        kwargs["object_hook"] = set_encoding

    try:
        value = json.loads(*args, **kwargs)
    except Exception:
        logger.error()
        value = {}

    return value


def dump_json(*args, **kwargs):
    if not kwargs:
        kwargs = {
            'indent': 4,
            'skipkeys': True,
            'sort_keys': True,
            'ensure_ascii': False
        }

    try:
        value = json.dumps(*args, **kwargs)
    except Exception:
        logger.error()
        value = ''

    return value


def set_encoding(dct):
    if isinstance(dct, dict):
        return dict((set_encoding(key), set_encoding(value)) for key, value in dct.items())

    elif isinstance(dct, list):
        return [set_encoding(element) for element in dct]

    elif isinstance(dct, six.string_types):
        return six.ensure_str(dct)

    else:
        return dct
