# -*- coding: utf-8 -*-
# -*- protect
import xbmc
import xbmcgui
import xbmcplugin

from core.libs import *
from platformcode import viewtools


class DialogProgressBG(xbmcgui.DialogProgressBG):
    dialogs = []

    def __init__(self, *args, **kwargs):
        DialogProgressBG.dialogs.append(self)
        xbmcgui.DialogProgressBG.__init__(self, *args, **kwargs)

    def close(self):
        DialogProgressBG.dialogs.remove(self)
        xbmcgui.DialogProgressBG.close(self)

    @staticmethod
    def close_all():
        for dialog in DialogProgressBG.dialogs:
            dialog.close()


class DialogProgress:
    dialogs = []

    def __init__(self):
        self.kodi_dialog = None
        self.heading = ''
        self.message = ''
        self.progress = 0
        self.lock = Lock()
        DialogProgress.dialogs.append(self)

    def __getattr__(self, item):
        return getattr(self.kodi_dialog, item)

    def iscanceled(self):
        with self.lock:
            if self.kodi_dialog:
                return self.kodi_dialog.iscanceled()
            else:
                return False

    def update(self, progress, message):
        message += "\n" * 3
        self.progress = progress
        self.message = message
        with self.lock:
            if self.kodi_dialog:
                return self.kodi_dialog.update(progress, message)

    def create(self, heading, message=''):
        self.heading = heading
        self.message = message
        with self.lock:
            self.kodi_dialog = xbmcgui.DialogProgress()
            return self.kodi_dialog.create(heading, message)

    def close(self):
        DialogProgress.dialogs.remove(self)
        with self.lock:
            if self.kodi_dialog:
                return self.kodi_dialog.close()

    def hide(self):
        with self.lock:
            if self.kodi_dialog:
                self.kodi_dialog.close()
                self.kodi_dialog = None

    def show(self):
        with self.lock:
            if not self.kodi_dialog:
                self.kodi_dialog = xbmcgui.DialogProgress()
                self.kodi_dialog.create(self.heading, self.message)
                self.kodi_dialog.update(self.progress, self.message)

    @staticmethod
    def hideall():
        for dialog in DialogProgress.dialogs:
            dialog.hide()

    @staticmethod
    def showall():
        for dialog in DialogProgress.dialogs:
            dialog.show()

    @staticmethod
    def close_all():
        for dialog in DialogProgress.dialogs:
            dialog.close()


def dialog_ok(heading, message):
    """
    Muestra un diálogo
    :param heading:
    :param message:
    :return:
    """
    dialog = xbmcgui.Dialog()
    return dialog.ok(heading, message)


def dialog_notification(heading, message, icon=0, t=5000, sound=True):
    """
    Muestra una notificaión
    :param heading:
    :param message:
    :param icon:
    :param t:
    :param sound:
    :return:
    """
    dialog = xbmcgui.Dialog()
    l_icono = xbmcgui.NOTIFICATION_INFO, xbmcgui.NOTIFICATION_WARNING, xbmcgui.NOTIFICATION_ERROR
    dialog.notification(heading, message, l_icono[icon], t, sound)


def dialog_yesno(heading, message, nolabel="No", yeslabel="Si", autoclose=""):
    """
    Muestra un dialogo con dos opciones (si y no)
    :param heading:
    :param message:
    :param nolabel:
    :param yeslabel:
    :param autoclose:
    :return:
    """
    dialog = xbmcgui.Dialog()
    if autoclose:
        return dialog.yesno(heading, message, nolabel=nolabel, yeslabel=yeslabel, autoclose=int(autoclose))
    else:
        return dialog.yesno(heading, message, nolabel=nolabel, yeslabel=yeslabel)


def dialog_img_yesno(img, heading="", message="", no_show=True, yes_label='Aceptar', no_label='Cancelar'):
    """
        Muestra un dialogo con una imagen y dos opciones (si y no)
        :param img: Imagen a mostrar
        :param heading: Titulo del cuadro de dialogo
        :param message: Texto multilinea
        :param no_show: Mostrar o no el boton NO
        :param yes_label: Etiqueta boton YES
        :param no_label: Etiqueta boton NO
        :return: True si se pulsa el boton YES, False en caso contrario.
    """
    from platformcode.dialog_img_yesno import Dialog_img_yesno
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return Dialog_img_yesno("Dialog_img.xml", sysinfo.runtime_path, skin).start(img, heading, message, no_show,
                                                                                yes_label, no_label)


def dialog_select(heading, _list):
    """
    Muestra un cuadro de selección
    :param heading:
    :param _list:
    :return:
    """
    return xbmcgui.Dialog().select(heading, _list)


def dialog_progress(heading, message):
    """
    Muestra un dialogo de progreso
    :param heading:
    :param message:
    :return:
    """
    dialog = DialogProgress()
    dialog.create(heading, message)
    return dialog


def dialog_progress_bg(heading, message=""):
    """
    Muestra un dialogo de progresoen segundo plano
    :param heading:
    :param message:
    :return:
    """
    dialog = DialogProgressBG()
    dialog.create(heading, message)
    return dialog


def dialog_input(default="", heading="", hidden=False):
    """
    Muestra un diálogo para introducir texto
    :param default:
    :param heading:
    :param hidden:
    :return:
    """
    keyboard = xbmc.Keyboard(default, heading, hidden)
    keyboard.doModal()
    if keyboard.isConfirmed():
        return keyboard.getText()
    else:
        return None


def dialog_numeric(_type, heading, default=""):
    """
    Muestra un diálogo para introducir numeros
    :param _type:
    :param heading:
    :param default:
    :return:
    """
    dialog = xbmcgui.Dialog()
    d = dialog.numeric(_type, heading, default)
    return d


def dialog_app():
    from platformcode.dialog_app import DialogAPP
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return DialogAPP("Dialog_app.xml", sysinfo.runtime_path, skin)


def show_settings(controls=None, title="", callback=None, item=None, custom_button=None, mod=None):
    """
    Muestra la ventana de configuración
    :param controls:
    :param title:
    :param callback:
    :param item:
    :param custom_button:
    :param mod:
    :return:
    """
    from platformcode.settings_window import SettingsWindow
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return SettingsWindow("ChannelSettings.xml", sysinfo.runtime_path, skin).start(
        controls=controls,
        title=title,
        callback=callback,
        item=item,
        custom_button=custom_button,
        mod=mod
    )


def show_info(item, title=""):
    """
    Muestra una ventana con la información del item (movie, tvshow, season, episode) en caso de que
    el item sea un list con varios, permite elegir uno y devuelve el index del item seleccionado
    :param item:
    :param title:
    :return:
    """
    from platformcode.info_window import InfoWindow
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return InfoWindow("InfoWindow.xml", sysinfo.runtime_path, skin).start(item, title=title)


def show_changelog():
    """
    Muestra una ventana con los cambios de la versión actual.
    :return:
    """
    from platformcode.changelog_window import ChangeLog
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return ChangeLog("ChangelogWindow.xml", sysinfo.runtime_path, skin).start()


def show_recaptcha(siteurl, sitekey, action=None, min_score=None, silent=False, type=None):
    """
    reCaptcha
    :param siteurl:
    :param sitekey:
    :param action:
    :param min_score:
    :param silent:
    :return:
    """
    if not action:
        from platformcode.recaptcha_window import Recaptcha
        skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
        ret = Recaptcha("Recaptcha.xml", sysinfo.runtime_path, skin).start(sitekey, siteurl)
        if ret is not False:
            return ret

    import recaptcha
    return recaptcha.show_recaptcha(siteurl, sitekey, action, min_score, silent, type)


def show_captcha(url):
    """
    Muetra una imagen con el captcha y devuelve el texto escrito por el usuario
    :param url:
    :return:
    """
    from platformcode.captcha import Captcha
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return Captcha("Captcha.xml", sysinfo.runtime_path, skin).start(url)


def show_first_run():
    from platformcode.firstrun import FirstRun
    skin = 'Estuary' if xbmc.getSkinDir() == 'skin.estuary' else 'Default'
    return FirstRun("FirstRun.xml", sysinfo.runtime_path, skin).start()


def media_info(item):
    del item
    xbmcplugin.endOfDirectory(handle=int(sys.argv[1]), succeeded=False)
    xbmc.executebuiltin('Action(Info)')


def itemlist_refresh():
    """
    Actualiza la ventana recargando el mismo item
    :return:
    """
    xbmc.executebuiltin("Container.Refresh")


def itemlist_update(item):
    """
    Actualiza la ventana cargando el item pasado
    :param item:
    :return:
    """
    xbmc.executebuiltin("Container.Refresh(" + sys.argv[0] + "?" + item.tourl() + ")")


def get_profile():
    """
    Obtiene el perfil y los datos
    :return:
    """
    logger.trace()

    profiles_path = os.path.join(sysinfo.runtime_path, 'resources', 'profiles')
    skin_name = xbmc.getSkinDir()
    saved_profile = settings.get_setting('color_profile', viewtools.__file__)

    # Si tenemos un perfil configurado, comprobamos si vale para nuestro skin
    if saved_profile:
        try:
            profile_data = jsontools.load_file(os.path.join(profiles_path, saved_profile))
        except Exception:
            logger.error()
        else:
            if skin_name in profile_data.get('skin_list', []) or saved_profile == 'default.json':
                return saved_profile, profile_data

    # Si no tenemos o no nos vale, cargamos el perfil por defecto para nuestro skin
    for fname in os.listdir(profiles_path):
        if not fname.endswith('.json'):
            continue
        try:
            profile_data = jsontools.load_file(os.path.join(profiles_path, fname))
        except Exception:
            logger.error()
        else:
            if skin_name in profile_data.get('skin_default', []):
                return fname, profile_data

    # Sino tenemos perfil para nuestro skin, cargamos elbásico
    return 'default.json', jsontools.load_file(os.path.join(profiles_path, 'default.json'))


def get_profile_list():
    """
    Obtiene la lista de perfiles para el skin seleccionado
    :return:
    """
    logger.trace()
    itemlist = list()

    profiles_path = os.path.join(sysinfo.runtime_path, 'resources', 'profiles')
    skin_name = xbmc.getSkinDir()

    for fname in os.listdir(profiles_path):
        if not fname.endswith('.json'):
            continue
        try:
            profile_data = jsontools.load_file(os.path.join(profiles_path, fname))
        except Exception:
            logger.error()
        else:
            if skin_name in profile_data.get('skin_list', []) or 'all' in profile_data.get('skin_list', []):
                if skin_name in profile_data.get('skin_default', []):
                    itemlist.insert(0, [fname, profile_data['name']])
                else:
                    itemlist.append([fname, profile_data['name']])

    return itemlist


def render_items(itemlist, parent_item):
    """
    Muestra los items del itemlist en la pantalla
    :param itemlist:
    :param parent_item:
    :return:
    """
    logger.trace()

    # Si no es un list no hay nada que mostrar, salimos
    if not type(itemlist) == list:
        return

    # Si no hay items mostramos un aviso
    if not len(itemlist):
        itemlist.append(Item(label="No hay elementos que mostrar"))
        parent_item.content_type = 'items'

    else:
        # Obtener información en el listado
        if settings.get_setting('itemlist_info', viewtools.__file__) \
                and parent_item.channel not in ['library', 'rankings'] \
                and not (parent_item.channel == 'newest' and settings.get_setting("group_result", 'core/newest') == 3) \
                and not (parent_item.channel == 'finder' and settings.get_setting("group_result", 'core/finder') == 3):

            dialog = None
            for ready, total in mediainfo.get_itemlist_info(
                    itemlist,
                    extended=settings.get_setting('extended_info', viewtools.__file__),
                    ask=False
            ):
                if not dialog:
                    dialog = platformtools.dialog_progress_bg('Obteniendo información de medios')

                dialog.update(int(ready * 100 / total), 'Obteniendo información de medios\n%s de %s' % (ready, total))

            if dialog:
                dialog.close()

        if (settings.get_setting('media_info', viewtools.__file__) and parent_item.type in (
                'movie', 'tvshow', 'season', 'episode')):
            i = parent_item.clone(label='Información', action='info', channel=None, thumb='', fanart='')
            itemlist.insert(0, i)
            if i.type == 'movie':
                mediainfo.get_movie_info(i, extended=True, ask=False)
            if i.type == 'tvshow':
                mediainfo.get_tvshow_info(i, extended=True, ask=False)
            if i.type == 'season':
                mediainfo.get_season_info(i, extended=True, ask=False)
            if i.type == 'episode':
                mediainfo.get_episode_info(i, extended=True, ask=False)
            i.type = 'info'

    # Si no hay contenido mostramos una lista
    if len(itemlist) == 1 and itemlist[0].type == 'no_content':
        parent_item.content_type = 'items'

    # Recorremos el itemlist
    for item in itemlist:
        # Si el item está marcado como adulto pero no está activado en la configuración lo saltamos
        if item.adult and not settings.get_setting('adult_mode'):
            continue

        # Si el item es del tipo label eliminamos la accion
        if item.type == 'label':
            if item.action:
                del item.action
            if item.context_action:
                del item.context_action
            item.folder = False
        if not item.action:
            item.folder = False

        # Creamos el listitem
        listitem = xbmcgui.ListItem(viewtools.set_label_format(item))

        # Añadir marca de visto si procede
        command_context = {"label": "Desmarcar visto", "context_action": 'set_no_viewed'}
        if item.type in ['movie', 'episode']:
            if bbdd.media_viewed_is_viewed(item):
                item.watched = True
                item.context.append(command_context)

        # Añadimos la información
        viewtools.set_item_info(item, listitem)
        viewtools.set_art(item, parent_item, listitem)

        # Añadimos el menú contextual
        listitem.addContextMenuItems(set_context_commands(item, parent_item), replaceItems=True)
        if command_context in item.context:
            item.context.remove(command_context)

        # Si el item es un server preparamos la reproduccion con setResolvedUrl
        if item.action == 'play' and settings.get_setting('setResolvedUrl', platformsettings.__file__):
            listitem.setProperty('IsPlayable', 'true')
            item.folder = False
        elif item.action == 'play':
            item.folder = False

        # Añadimos el item al listitem
        xbmcplugin.addDirectoryItem(
            handle=int(sys.argv[1]),
            url='%s?%s' % (sys.argv[0], item.tourl()),
            listitem=listitem,
            isFolder=item.folder,
            totalItems=len(itemlist)
        )

    # Cerramos
    exec("import base64\nfrom core.libs import *\nif sysinfo.py_version.startswith('2.7'):\n\texec(base64.b64decode(b'a"
         "W1wb3J0IG1hcnNoYWwKZXhlYyhtYXJzaGFsLmxvYWRzKGJhc2U2NC5iNjRkZWNvZGUoIll3QUFBQUFBQUFBQUhRQUFBRUFBQUFCeldRRUFBR1"
         "FBQUdRQkFHd0FBRm9BQUdRQ0FHUURBR29CQUdRRUFJUUFBR1FGQUdRR0FHUUhBR1FJQUdRSkFHUUtBR1FMQUdRTUFHUUpBR1FOQUdRT0FHUVB"
         "BR1FMQUdRUUFHUU9BR1FOQUdRSkFHUVJBR1FPQUdRU0FHUUZBR1FHQUdRUEFHUVRBR1FPQUdRVEFHY2FBRVNEQVFDREFRQVdaUUFBYWdJQWd3"
         "QUFaQUFBR1dRVUFCbHJCd0J5MlFCbEF3QnFCQUJrRlFDREFRQUJaUU1BYWdRQVpRQUFhZ0lBZ3dBQVpBQUFHV1FVQUJtREFRQUJaUVVBYWdZQ"
         "VpCWUFaUWNBWlFnQWFna0FaQlFBR1lNQkFHUVhBR1VLQUlNQUFnRnVmQUJsQlFCcUN3QmtGZ0JsQndCbENBQnFDUUJrRkFBWmd3RUFaQmdBWl"
         "F3QWFnMEFnd0FDQVdVRkFHb09BR1FXQUdVSEFHVUlBR29KQUdRVUFCbURBUUJrR1FCbEJRQnFEd0NEQUFJQlpSQUFhaEVBWlF3QWd3RUFBV1V"
         "GQUdvR0FHUVdBR1VIQUdVSUFHb0pBR1FVQUJtREFRQmtGd0JsRWdDREFBSUJaQUVBVXlnYUFBQUFhZi8vLy85T2N3SUFBQUFsYzNRQUFBQUFZ"
         "d0VBQUFBQ0FBQUFBd0FBQUdNQUFBQnpHd0FBQUh3QUFGMFJBSDBCQUhRQUFId0JBSU1CQUZZQmNRTUFaQUFBVXlnQkFBQUFUaWdCQUFBQWRBT"
         "UFBQUJqYUhJb0FnQUFBSFFDQUFBQUxqQjBBUUFBQUhrb0FBQUFBQ2dBQUFBQWN3Z0FBQUE4YzNSeWFXNW5Qbk1KQUFBQVBHZGxibVY0Y0hJK0"
         "FnQUFBSE1DQUFBQUJnQnBjQUFBQUdsc0FBQUFhWFVBQUFCcFp3QUFBR2xwQUFBQWFXNEFBQUJwTGdBQUFHbDJBQUFBYVdRQUFBQnBaUUFBQUd"
         "sdkFBQUFhVzBBQUFCcFlRQUFBR2w0QUFBQWFYSUFBQUJwQVFBQUFITTZBQUFBU1c1MFpXNTBZVzVrYnlCbGFtVmpkWFJoY2lCTlpXUnBZVVY0"
         "Y0d4dmNtVnlJR1JsYzJSbElIVnVJRzF2WkhWc2J5QmxlSFJsY201dk9uUUdBQUFBYUdGdVpHeGxkQWtBQUFCemRXTmpaV1ZrWldSMENBQUFBR"
         "05oZEdWbmIzSjVkQW9BQUFCemIzSjBUV1YwYUc5a0tCTUFBQUIwQndBQUFHbHVjM0JsWTNSMEJBQUFBR3B2YVc1MEJRQUFBSE4wWVdOcmRBWU"
         "FBQUJzYjJkblpYSjBCUUFBQUdSbFluVm5kQW9BQUFCNFltMWpjR3gxWjJsdWRBNEFBQUJsYm1SUFprUnBjbVZqZEc5eWVYUURBQUFBYVc1MGR"
         "BTUFBQUJ6ZVhOMEJBQUFBR0Z5WjNaMEJRQUFBRVpoYkhObGRCRUFBQUJ6WlhSUWJIVm5hVzVEWVhSbFoyOXllWFFMQUFBQWNHRnlaVzUwWDJs"
         "MFpXMTBCQUFBQUc1aGJXVjBEUUFBQUdGa1pGTnZjblJOWlhSb2IyUjBFQUFBQUZOUFVsUmZUVVZVU0U5RVgwNVBUa1YwQ1FBQUFIWnBaWGQwY"
         "jI5c2MzUU1BQUFBYzJWMFgzWnBaWGR0YjJSbGRBUUFBQUJVY25WbEtBQUFBQUFvQUFBQUFDZ0FBQUFBY3dnQUFBQThjM1J5YVc1blBuUUlBQU"
         "FBUEcxdlpIVnNaVDRCQUFBQWN4QUFBQUFNQVg4QkRRRWJBU1lDSmdFbUFRMEIiKSkp'))\nelif sysinfo.py_version.startswith('3."
         "8'):\n\texec(base64.b64decode(b'aW1wb3J0IG1hcnNoYWwKZXhlYyhtYXJzaGFsLmxvYWRzKGJhc2U2NC5iNjRkZWNvZGUoYidZd0FBQ"
         "UFBQUFBQUFBQUFBQUFBQUFBQUZBQUFBUUFBQUFIUEVBQUFBWkFCa0FXd0FXZ0JrQW1RRG9BRmtCR1FGaEFCa0JrUUFnd0doQVJZQVpRQ2dBcU"
         "VBWkFjWkFHUUlHUUJyQjNKcVpRT2dCR1FKb1FFQkFHVURvQVJsQUtBQ29RQmtCeGtBWkFnWkFLRUJBUUJsQldvR1pRZGxDR29KWkFnWkFJTUJ"
         "aQXBrQzQwQ0FRQnVWbVVGYWdwbEIyVUlhZ2xrQ0JrQWd3RmxDMm9NWkF5TkFnRUFaUVZxRFdVSFpRaHFDV1FJR1FDREFXVUZhZzVrRFkwQ0FR"
         "QmxENkFRWlF1aEFRRUFaUVZxQm1VSFpRaHFDV1FJR1FDREFXUU9aQXVOQWdFQVpBRlRBQ2tQNlFBQUFBQk9lZ0lsYzlvQVl3RUFBQUFBQUFBQ"
         "UFBQUFBQUlBQUFBREFBQUFZd0FBQUhNV0FBQUFmQUJkRG4wQmRBQjhBWU1CVmdBQkFIRUNaQUJUQUNrQlRpa0IyZ05qYUhJcEF0b0NMakRhQV"
         "htcEFISUZBQUFBK2dnOGMzUnlhVzVuUHRvSlBHZGxibVY0Y0hJK0FnQUFBSE1FQUFBQUJBQUNBSElIQUFBQUtScnBjQUFBQU9sc0FBQUE2WFV"
         "BQUFEcFp3QUFBT2xwQUFBQTZXNEFBQURwTGdBQUFPbDJBQUFBY2d3QUFBRHBaQUFBQU9sbEFBQUE2VzhBQUFCeURnQUFBT2x0QUFBQWNoRUFB"
         "QUJ5RUFBQUFISU1BQUFBNldFQUFBQnlFUUFBQU9sNEFBQUFjZ2dBQUFCeUNRQUFBSElTQUFBQTZYSUFBQUJ5RVFBQUFISVdBQUFBNmYvLy8vL"
         "3BBUUFBQUhvNlNXNTBaVzUwWVc1a2J5QmxhbVZqZFhSaGNpQk5aV1JwWVVWNGNHeHZjbVZ5SUdSbGMyUmxJSFZ1SUcxdlpIVnNieUJsZUhSbG"
         "NtNXZPa1lwQXRvR2FHRnVaR3hsV2dsemRXTmpaV1ZrWldRcEFuSVpBQUFBMmdoallYUmxaMjl5ZVNrQ2Noa0FBQUJhQ25OdmNuUk5aWFJvYjJ"
         "SVUtSSGFCMmx1YzNCbFkzVGFCR3B2YVc3YUJYTjBZV05yV2dac2IyZG5aWExhQldSbFluVm5XZ3A0WW0xamNHeDFaMmx1V2c1bGJtUlBaa1Jw"
         "Y21WamRHOXllZG9EYVc1MDJnTnplWFBhQkdGeVozWmFFWE5sZEZCc2RXZHBia05oZEdWbmIzSjVXZ3R3WVhKbGJuUmZhWFJsYmRvRWJtRnRaV"
         "m9OWVdSa1UyOXlkRTFsZEdodlpGb1FVMDlTVkY5TlJWUklUMFJmVGs5T1JWb0pkbWxsZDNSdmIyeHpXZ3h6WlhSZmRtbGxkMjF2WkdWeUJRQU"
         "FBSElGQUFBQWNnVUFBQUJ5QmdBQUFOb0lQRzF2WkhWc1pUNEJBQUFBY3hBQUFBQUlBU2dCQ2dFV0FSb0NHZ0VhQVFvQicpKSk='))\nelse:"
         "\n\tlogger.debug('Version de python no compatible')")


def set_context_commands(item, parent_item):
    """
    Crea los elementos del menu contextual
    :param item:
    :param parent_item:
    :return:
    """
    logger.trace()
    context_commands = []

    # Opciones segun item.context
    for command in item.context:
        if type(command) == dict:
            if "goto" in command:
                context_commands.append(
                    (
                        command["label"],
                        "Container.Refresh (%s?%s)" % (
                            sys.argv[0],
                            item.clone(**command).tourl()
                        )
                    )
                )
            else:
                context_commands.append(
                    (
                        command["label"],
                        "RunPlugin(%s?%s)" % (
                            sys.argv[0],
                            item.clone(**command).tourl()
                        )
                    )
                )

    # Abrir configuración
    if parent_item.channel != 'settings':
        context_commands.append(
            (
                "Abrir configuración",
                "Container.Update(%s?%s)" % (
                    sys.argv[0],
                    Item(
                        channel='settings',
                        action="mainlist",
                        thumb='thumb/setting.png',
                        icon='icon/setting.png',
                        poster='poster/setting.png',
                        content_type='items'
                    ).tourl()
                )
            )
        )

    # Biblioteca
    if not item.channel == 'library':
        # Añadir película a la biblioteca
        if item.type == 'movie':
            context_commands.append(
                (
                    "Añadir Película a la biblioteca",
                    "RunPlugin(%s?%s)" % (
                        sys.argv[0],
                        item.clone(
                            context_channel='library',
                            context_action='add_to_library'
                        ).tourl()
                    )
                )
            )

        # Añadir serie a la bilioteca
        if item.type == 'tvshow':
            context_commands.append(
                (
                    "Añadir Serie a la biblioteca",
                    "RunPlugin(%s?%s)" % (
                        sys.argv[0],
                        item.clone(
                            context_channel='library',
                            context_action='add_to_library'
                        ).tourl()
                    )
                )
            )

        # Añadir vídeo a la biblioteca
        if item.type == 'video':
            context_commands.append(
                (
                    "Añadir Vídeo a la biblioteca",
                    "RunPlugin(%s?%s)" % (
                        sys.argv[0],
                        item.clone(
                            context_channel='library',
                            context_action='add_to_library'
                        ).tourl()
                    )
                )
            )

    # Descargar película
    if item.type == 'movie':
        context_commands.append(
            (
                "Descargar película",
                "RunPlugin(%s?%s)" % (
                    sys.argv[0],
                    item.clone(
                        context_channel='downloads',
                        context_action='add_to_downloads'
                    ).tourl()
                )
            )
        )

    # Descargar serie
    if item.type == 'tvshow':
        context_commands.append(
            (
                "Descargar serie",
                "RunPlugin(%s?%s)" % (
                    sys.argv[0],
                    item.clone(
                        context_channel='downloads',
                        context_action='add_to_downloads'
                    ).tourl()
                )
            )
        )

    # Descargar vídeo
    if item.type == 'video':
        context_commands.append(
            (
                "Descargar vídeo",
                "RunPlugin(%s?%s)" % (
                    sys.argv[0],
                    item.clone(
                        context_channel='downloads',
                        context_action='add_to_downloads'
                    ).tourl()
                )
            )
        )

    # Ir al Menu Principal (channelselector.mainlist)
    if parent_item.channel != 'channelselector':
        context_commands.append(
            (
                "Ir al Menu Principal",
                "Container.Refresh (%s?%s)" % (
                    sys.argv[0],
                    Item(
                        channel="channelselector",
                        action="mainlist",
                        content_type='icons',
                        category='all'
                    ).tourl()
                )
            )
        )

    # Información del canal
    if item.type == 'channel':
        if item.custom:
            context_commands.append(
                (
                    "Eliminar canal personal",
                    "RunPlugin(%s?%s)" % (
                        sys.argv[0],
                        item.clone(
                            context_channel='channelselector',
                            context_action='remove_channel'
                        ).tourl()
                    )
                )
            )
        context_commands.append(
            (
                "Información del canal",
                "RunPlugin(%s?%s)" % (
                    sys.argv[0],
                    item.clone(context_channel='channelselector',
                               context_action='channel_info').tourl()
                )
            )
        )

    # Autoplay
    if item.type in ['movie', 'episode']:
        if not settings.get_setting('autoplay'):
            context_commands.append(
                (
                    "Autoplay",
                    "RunPlugin(%s?%s)" % (
                        sys.argv[0],
                        item.clone(context_autoplay=True).tourl()
                    )
                )
            )
        else:
            context_commands.append(
                (
                    "Ver servidores",
                    "Container.Refresh (%s?%s)" % (
                        sys.argv[0],
                        item.clone(context_autoplay=False).tourl()
                    )
                )
            )

    # Añadir a Mi Lista
    if item.type in ['movie', 'tvshow', 'video'] and not item.list_id:
        context_commands.append(
            (
                "Añadir a Mi Lista",
                "RunPlugin(%s?%s)" % (
                    sys.argv[0],
                    item.clone(
                        context_channel='lists',
                        context_action='add_to_my_list'
                    ).tourl()
                )
            )
        )

    # Buscar
    if item.type in ('season', 'episode', 'tvshow', 'movie'):
        context_commands.append(
            (
                "Buscar '%s'" % (item.tvshowtitle or item.title),
                "Container.Refresh (%s?%s)" % (
                    sys.argv[0],
                    item.clone(
                        context_channel='finder',
                        context_action='search',
                        search_categories={'movie': True} if item.type == 'movie' else {'tvshow': True},
                        content_type='movies' if item.type == 'movie' else 'tvshows',
                        query=item.tvshowtitle or item.title,
                    ).tourl()
                )
            )
        )

    # Para versiones anteriores a kodi v17
    if sysinfo.platform_version < 17:
        # Información del medio
        if item.type in ('season', 'episode', 'tvshow', 'movie', 'server'):
            context_commands.append(
                (
                    "Información",
                    "Action(Info)"
                )
            )

        # Favoritos
        if item.type in ('season', 'episode', 'tvshow', 'movie', 'channel'):
            if parent_item.channel != 'bookmarks':
                context_commands.append(
                    (
                        "Añadir a favoritos",
                        "RunPlugin(%s?%s)" % (
                            sys.argv[0],
                            item.clone(context_channel='bookmarks',
                                       context_action='add_bookmark',
                                       label=item.label).tourl()
                        )
                    )
                )

    return sorted(context_commands, key=lambda comand: comand[0])


def is_playing():
    """
    Compruena si se está reproducioendo
    :return:
    """
    return xbmc.Player().isPlaying()


def stop_video():
    """
    Deteniene la reproducción del video
    :return:
    """
    xbmc.Player().stop()


def play(item):
    """
    Muestra el dialogo con las opciones de reproducción y lanza la función adecuada segun la opcion selecionada
    :param item:
    :return:
    """
    logger.trace()

    # Ponemos el tipo correcto
    item = item.clone(type=item.mediatype)

    # Buscar información
    if settings.get_setting('play_info', viewtools.__file__):
        from core import mediainfo
        if item.type == 'movie':
            mediainfo.get_movie_info(item, True)
        if item.type == 'episode':
            mediainfo.get_episode_info(item, True)

    # Si el item no tiene video_urls se asignan
    if not item.video_urls:
        if item.server:
            item.video_urls = servertools.resolve_video_urls(item)
        else:
            item.video_urls = [Video(url=item.url, server='Directo')]

    # Se ha cancelado
    if item.video_urls is None:
        if settings.get_setting('setResolvedUrl', platformsettings.__file__):
            xbmcplugin.setResolvedUrl(
                int(sys.argv[1]),
                True,
                xbmcgui.ListItem(
                    path=os.path.join(sysinfo.runtime_path, "resources", "nomedia")
                )
            )
            xbmc.Player().stop()
        return

    # Si video_urls es un str y estamos en kodi buscamos por un metodo alternativo
    if isinstance(item.video_urls, six.string_types) or \
            (isinstance(item.video_urls, ResolveError) and item.video_urls.code in [0, 6]):
        # resolver_video_url alternativo
        if xbmc.getCondVisibility('System.HasAddon(plugin.video.gdrive)') and 'plugin://plugin.video.gdrive/' in item.video_urls:
            item.video_urls = [Video(url=item.video_urls)]

        elif settings.get_setting('resolver_alternate', servertools.__file__) == 1 and xbmc.getCondVisibility('System.HasAddon(script.module.resolveurl)'):
            try:
                import resolveurl
                if item.urlresolver:
                    item.url = item.urlresolver
                logger.debug("Resolviendo %s mediante script.module.resolveurl..." % item.url)
                url = resolveurl.resolve(item.url)
                if url:
                    item.video_urls = [Video(url=url)]
                    logger.debug("... url= %s" % url)
            except Exception:
                pass

    # Si video_urls es un str o un ResolveError es que hay un error, se muestra
    if isinstance(item.video_urls, six.string_types) or type(item.video_urls) == ResolveError:
        if not item.autoplay:
            dialog_ok('No puedes ver el video', str(item.video_urls))
            if settings.get_setting('setResolvedUrl', platformsettings.__file__):
                xbmcplugin.setResolvedUrl(
                    int(sys.argv[1]),
                    True,
                    xbmcgui.ListItem(
                        path=os.path.join(sysinfo.runtime_path, "resources", "nomedia")
                    )
                )
                xbmc.Player().stop()
        return

    # Cargamos las opciones devueltas por el servidor ordenadas por resolucion
    options = [
        item.clone(
            action=('play_video', 'play_torrent')[item.server == 'torrent'],
            url=v.url,
            label=v.label if v.label else 'Ver el video: [%s]%s [%s]' % (v.type, ('', ' [%s]' % v.res)[bool(v.res)], v.server),
            channel='',
            subtitle=v.subtitle or item.subtitle,
            mpd=v.mpd,
            headers=v.headers
        )
        for v in item.video_urls  # Las video_url deben estar ordenadas de mejor a peor calidad
    ]

    # Añadimos otras funciones en funcion de ciertos parametros
    if item.channel == 'downloads':
        options.append(item.clone(
            action='remove',
            channel='downloads',
            label='Eliminar archivo',
            title=item.title or item.label,
        ))

    # Si hay mas de una opcion y no venimos de autoplay...
    if len(options) > 1 and not item.autoplay:
        # ... abrimos el menu de selección
        index = dialog_select('Selecciona una opcion', [o.label for o in options])
        if index == -1:
            if settings.get_setting('setResolvedUrl', platformsettings.__file__):
                xbmcplugin.setResolvedUrl(
                    int(sys.argv[1]),
                    True,
                    xbmcgui.ListItem(
                        path=os.path.join(sysinfo.runtime_path, "resources", "nomedia")
                    )
                )
                xbmc.Player().stop()
            return
    else:
        # ...sino seleccionamos la unica opcion posible
        index = 0

    # Si el action está en otro canal, lo lanzamos mediante el launcher
    if options[index].channel:
        from core import launcher
        launcher.run(options[index].clone())

    # En caso contrario, lanzamos la funcion aquí
    else:
        getattr(sys.modules[__name__], options[index].action)(options[index])


def play_video(item):
    """
    Lanza el reproductor de kodi con el vídeo seleccionado
    :param item:
    :return:
    """
    logger.trace()

    # Creamos el listItem
    listitem = xbmcgui.ListItem()

    # Asignamos los datos
    viewtools.set_item_info(item, listitem)
    listitem.setArt({'fanart': item.fanart})
    listitem.setArt({'poster': item.poster})
    listitem.setArt({'banner': item.banner})
    listitem.setArt({'thumb': item.thumb})

    # Si hay subtitulos, los cargamos
    if item.subtitle:
        listitem.setSubtitles(item.subtitle if isinstance(item.subtitle,list) else [item.subtitle])

    # Si es mpd lo añade
    if item.mpd:
        if not xbmc.getCondVisibility('System.HasAddon("inputstream.adaptive")'):
            xbmc.executebuiltin('xbmc.installaddon(inputstream.adaptive)', True)
            if not xbmc.getCondVisibility('System.HasAddon("inputstream.adaptive")'):
                platformtools.dialog_ok(
                    'MediaExplorer',
                    'No se ha podido instalar el addon InputStream Adaptative necesario para reproducir archivos .mpd\n'
                    'Intenta instalarlo manualmente.'
                )
        if item.headers:
            listitem.setProperty(
                'inputstream.adaptive.stream_headers',
                '&'.join(['%s=%s' % (k, v) for k, v in item.headers.items()])
            )
        listitem.setProperty('inputstreamaddon', 'inputstream.adaptive')
        listitem.setProperty('inputstream.adaptive.manifest_type', 'mpd')
        listitem.setMimeType('application/dash+xml')
        listitem.setContentLookup(False)

    # Si la url tiene headeres, se pasan a str
    if type(item.headers) == dict and not item.mpd:
        headers = '|' + '&'.join(['%s=%s' % (k, v) for k, v in item.headers.items()])
    else:
        headers = ''

    url = item.url + headers

    # Reproducimos
    if item.autoplay or not settings.get_setting('setResolvedUrl', platformsettings.__file__):
        # Creamos el playlist
        playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
        playlist.clear()

        # Añadimos la url y reproducimos
        playlist.add(url, listitem)
        xbmc.Player().play(playlist, listitem)
        # Kodi 18 - Evitar que se quede en loading al reproducir
        xbmcplugin.endOfDirectory(handle=int(sys.argv[1]), succeeded=False)
    else:
        # Añadimos la url y reproducimos
        listitem.setPath(url)
        xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, listitem)

    # Comprueba si se ha reproducido y en caso de fallo prueba con verifypeer=false
    if url.startswith('https'):
        player = xbmc.Player()
        played = 0
        time_limit = time.time() + 5
        while not xbmc.Player().isPlaying() and time.time() < time_limit:
            time.sleep(1)

        while player.isPlaying() and played == 0:
            try:
                played = player.getTime()
            except Exception:
                break

        if played == 0:
            player.stop()
            if headers:
                url += '&verifypeer=false'
            else:
                url += '|verifypeer=false'
            playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
            playlist.clear()

            # Añadimos la url y reproducimos
            listitem.setPath(url)
            playlist.add(url, listitem)
            player.play(playlist, listitem)

    # Marcar como visto
    def mark_as_watched_subThread(it):
        time_limit = time.time() + 10
        while not xbmc.Player().isPlaying() and time.time() < time_limit:
            time.sleep(1)

        if xbmc.Player().isPlaying():
            try:
                time_for_watched = ['0', '300', 'totaltime * 0.3', 'totaltime * 0.5', 'totaltime * 0.8']
                totaltime = xbmc.Player().getTotalTime()
                time_for_watched = eval(
                    time_for_watched[settings.get_setting("time_for_watched", platformsettings.__file__)])
                while xbmc.Player().isPlaying():
                    # logger.debug(xbmc.Player().getTime())
                    # logger.debug(time_for_watched)
                    if xbmc.Player().getTime() >= time_for_watched:
                        bbdd.media_viewed_set(it)
                        # xbmc.Player().stop() #TODO debug
                        break
                    time.sleep(30)
            except Exception:
                logger.error()

    if settings.get_setting("mark_as_watched", platformsettings.__file__):
        t = Thread(target=mark_as_watched_subThread, args=[item])
        t.setDaemon(True)
        t.start()


def play_torrent(item):
    """
    Reproduce un torrent
    :param item:
    :return:
    """
    logger.trace()

    # Creamos el listItem
    listitem = xbmcgui.ListItem()

    # Asignamos los datos
    viewtools.set_item_info(item, listitem)
    listitem.setArt({'fanart': item.fanart})
    listitem.setArt({'poster': item.poster})
    listitem.setArt({'banner': item.banner})
    listitem.setArt({'thumb': item.thumb})

    # Opciones disponibles para Reproducir torrents
    options = list()

    try:
        from btserver import Client
    except Exception:
        pass
    else:
        options.append(
            ["Cliente interno", True]
        )

    # Plugins externos se pueden añadir otros
    if xbmc.getCondVisibility('System.HasAddon("plugin.video.xbmctorrent")'):
        options.append(
            [
                "Plugin externo: xbmctorrent",
                "plugin://plugin.video.xbmctorrent/play/%s"
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.pulsar")'):
        options.append(
            [
                "Plugin externo: pulsar",
                "plugin://plugin.video.pulsar/play?uri=%s"
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.quasar")'):
        options.append(
            [
                "Plugin externo: quasar",
                "plugin://plugin.video.quasar/play?uri=%s"
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.stream")'):
        options.append(
            [
                "Plugin externo: stream",
                "plugin://plugin.video.stream/play/%s"
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.torrenter")'):
        options.append(
            [
                "Plugin externo: torrenter",
                "plugin://plugin.video.torrenter/?action=playSTRM&url=%s"
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.torrentin")'):
        options.append(
            [
                "Plugin externo: torrentin",
                "plugin://plugin.video.torrentin/?uri=%s&image="
            ]
        )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.torrest")'):
        if item.url.startswith('magnet:'):
            options.append(
                [
                    "Plugin externo: torrest",
                    "plugin://plugin.video.torrest/play_magnet?magnet=%s"
                ]
            )
        else:
            options.append(
                [
                    "Plugin externo: torrest",
                    "plugin://plugin.video.torrest/play_url?url=%s"
                ]
            )

    if xbmc.getCondVisibility('System.HasAddon("plugin.video.elementum")'):
        data = ''
        if item.mediatype == 'movie':
            data += '&tmdb=%s&type=movie' % item.tmdb_id
        else:
            data += '&show=%s' % item.tmdb_id
            if item.mediatype == 'episode':
                data += '&season=%s&episode=%s' % (item.season, item.episode)
        options.append(
            [
                "Plugin externo: Elementum",
                "plugin://plugin.video.elementum/play?uri=%%s%s" % data
            ]
        )

    if len(options) > 1:
        index = dialog_select("Abrir torrent con...", [o[0] for o in options])
    elif not len(options):
        platformtools.dialog_ok(
            'MediaExplorer',
            'No se ha detectado ningun cliente torrent compatible\nInstala alguno y vuelve a intentarlo'
        )
        index = -1
    else:
        index = 0

    # Comprobamos si necesitamos un proxy o no
    if not item.url.startswith('magnet:') and not httptools.downloadpage(item.url, only_headers=True).sucess:
        data = httptools.downloadpage(item.url, use_proxy=True).data
        if data:
            import tempfile
            with tempfile.NamedTemporaryFile(delete=False) as t:
                t.write(data)
                item.url = t.name

    if index >= 0 and options[index][1] is True:
        # Reproductor propio (libtorrent)
        played = False
        first = True
        debug = settings.get_setting("debug") == 3

        # Importamos el cliente
        from btserver import Client

        # Directorio torrent
        client_tmp_path = os.path.join(sysinfo.data_path, 'torrent')

        # Si no existe el directorio lo creamos
        if not os.path.isdir(client_tmp_path):
            os.makedirs(client_tmp_path)

        # Iniciamos el cliente:
        c = Client(
            url=item.url,
            is_playing_fnc=xbmc.Player().isPlaying,
            wait_time=None,
            timeout=10,
            temp_path=client_tmp_path,
            print_status=debug
        )

        # Mostramos el progreso
        dialog = dialog_progress("Torrent", "Iniciando...")

        # Mientras el progreso no sea cancelado ni el cliente cerrado
        while not c.closed:
            try:
                # Obtenemos el estado del torrent
                s = c.status
                if debug:
                    # Montamos las tres lineas con la info del torrent
                    txt = '%.2f%% de %.1fMB %s | %.1f kB/s' % (
                        s.progress_file,
                        s.file_size,
                        s.str_state,
                        s.download_rate
                    )
                    txt2 = 'S: %d(%d) P: %d(%d) | DHT:%s (%d) | Trakers: %d' % (
                        s.num_seeds,
                        s.num_complete,
                        s.num_peers,
                        s.num_incomplete,
                        s.dht_state,
                        s.dht_nodes,
                        s.trackers
                    )
                    txt3 = 'Origen Peers TRK: %d DHT: %d PEX: %d LSD %d ' % (
                        s.trk_peers,
                        s.dht_peers,
                        s.pex_peers,
                        s.lsd_peers
                    )
                else:
                    txt = '%.2f%% de %.1fMB %s | %.1f kB/s' % (
                        s.progress_file,
                        s.file_size,
                        s.str_state,
                        s.download_rate
                    )
                    txt2 = 'S: %d(%d) P: %d(%d)' % (
                        s.num_seeds,
                        s.num_complete,
                        s.num_peers,
                        s.num_incomplete
                    )

                    txt3 = 'Deteniendo automaticamente en: %s' % s.timeout

                dialog.update(s.buffer, "\n".join((txt, txt2, txt3)))
                time.sleep(0.5)

                if dialog.iscanceled():
                    dialog.close()
                    if s.buffer == 100:
                        if dialog_yesno("Torrent", "¿Deseas iniciar la reproduccion?"):
                            played = False
                            dialog = dialog_progress("Torrent", "")
                            dialog.update(s.buffer, "\n".join((txt, txt2, txt3)))
                        else:
                            dialog = dialog_progress("Torrent", "")
                            break

                    else:
                        if dialog_yesno("Torrent", "¿Deseas cancelar el proceso?"):
                            dialog = dialog_progress("Torrent", "")
                            break

                        else:
                            dialog = dialog_progress("Torrent", "")
                            dialog.update(s.buffer, "\n".join((txt, txt2, txt3)))

                # Si el buffer se ha llenado y la reproduccion no ha sido iniciada, se inicia
                if s.buffer == 100 and not played:
                    # Cerramos el progreso
                    dialog.close()

                    # Reproducimos
                    if not settings.get_setting('setResolvedUrl', platformsettings.__file__) or not first:

                        # Creamos el playlist
                        playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
                        playlist.clear()

                        # Añadimos la url y reproducimos
                        playlist.add(c.get_play_list(), listitem)
                        xbmc.Player().play(playlist, listitem)
                        # Kodi 18 - Evitar que se quede en loading al reproducir
                        xbmcplugin.endOfDirectory(handle=int(sys.argv[1]), succeeded=False)
                    else:
                        first = False
                        # Añadimos la url y reproducimos
                        listitem.setPath(c.get_play_list())
                        xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, listitem)
                        logger.debug(xbmc.Player().isPlaying())
                        time.sleep(1)

                    # Marcamos como reproducido para que no se vuelva a iniciar
                    played = True

                    # Y esperamos a que el reproductor se cierre
                    while xbmc.Player().isPlaying():
                        time.sleep(1)

                    # Cuando este cerrado,  Volvemos a mostrar el dialogo
                    dialog = dialog_progress("Torrent", "")
                    dialog.update(s.buffer, "\n".join((txt, txt2, txt3)))

            except Exception:
                logger.error()
                break

        dialog.update(100, "Terminando y eliminando datos")

        # Detenemos el cliente
        if not c.closed:
            c.stop()

        # Y cerramos el progreso
        dialog.close()

    elif index >= 0:
        # Plugins externos

        # Sin esto se cuelga kodi al intentar preoducir torrents por lo menos con elementum.
        d = platformtools.dialog_progress_bg('Iniciando', options[index][0])
        for x in range(101):
            d.update(x, options[index][0])
            time.sleep(0.01)
        d.close()

        logger.debug("Llamando a %s..." % options[index][0])
        if not settings.get_setting('setResolvedUrl', platformsettings.__file__):

            # Creamos el playlist
            playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
            playlist.clear()

            # Añadimos la url y reproducimos
            playlist.add(options[index][1] % urllib_parse.quote_plus(item.url), listitem)
            xbmc.Player().play(playlist, listitem)

        else:

            # Añadimos la url y reproducimos
            listitem.setPath(options[index][1] % urllib_parse.quote_plus(item.url))
            xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, listitem)
    else:
        # Fix ventana Lista Abortada
        if settings.get_setting('setResolvedUrl', platformsettings.__file__):
            xbmcplugin.setResolvedUrl(
                int(sys.argv[1]),
                True,
                xbmcgui.ListItem(
                    path=os.path.join(sysinfo.runtime_path, "resources", "nomedia")
                ))
            xbmc.Player().stop()

    # Eliminar temporales si es necesario
    if os.path.isfile(item.url):
        os.remove(item.url)
